
--=|The Database Diagram.|=--

--  
--                    pd_vendors     pd_roots
--                            1       1
--                            |       |            
--                            M       M
--   pd_tags                 pd_projects
--    1   1                   1  1  1 1
--    |   +-------+           |  |  | +--------+
--    M           M           M  |  M          M
--   pd_subtags  pd_project_tags | pd_files   pd_project_components
--           1                   |       1   
--           +----+              |       |
--                M              M       M
--               pd_project_subtags     pd_lines
--                                      

--=||=--
-- ===========================================================================
-- ===========================================================================
--     C R E A T E S
-- ===========================================================================
-- ===========================================================================

--=|Create the tables.|=--

    CREATE TABLE IF NOT EXISTS
      pd_vendors
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL, 
          description                   TEXT
        )
;
    CREATE TABLE IF NOT EXISTS
      pd_roots
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL, 
          path                          TEXT NOT NULL, 
          description                   TEXT
        )
;
    CREATE TABLE IF NOT EXISTS
      pd_tags
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL, 
          description                   TEXT
        )
;
    CREATE TABLE IF NOT EXISTS
      pd_projects
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL, 
          dir                           TEXT, 
          start_up                      TEXT, 
          version                       TEXT, 
          title                         TEXT, 
          description                   TEXT, 
          authors                       TEXT, 
          tab_size                      TEXT, 
          language                      TEXT, 
          packager                      TEXT,

          pd_vendors_fk                 INTEGER NOT NULL,
          pd_roots_fk                   INTEGER NOT NULL,

          FOREIGN KEY ( pd_vendors_fk )
            REFERENCES  pd_vendors( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE,

          FOREIGN KEY ( pd_roots_fk )
            REFERENCES  pd_roots( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_vendors_fk___pd_projects
              ON pd_projects( pd_vendors_fk )
;
    CREATE INDEX pd_roots_fk___pd_projects
              ON pd_projects( pd_roots_fk )
;
    CREATE TABLE IF NOT EXISTS
      pd_subtags
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL, 

          pd_tags_fk                    INTEGER NOT NULL,

          FOREIGN KEY ( pd_tags_fk )
            REFERENCES  pd_tags( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_tags_fk___pd_subtags
              ON pd_subtags( pd_tags_fk )
;
    CREATE TABLE IF NOT EXISTS
      pd_project_tags
        (
          id                            INTEGER PRIMARY KEY,

          pd_tags_fk                    INTEGER NOT NULL,
          pd_projects_fk                INTEGER NOT NULL,

          FOREIGN KEY ( pd_tags_fk )
            REFERENCES  pd_tags( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE,

          FOREIGN KEY ( pd_projects_fk )
            REFERENCES  pd_projects( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_tags_fk___pd_project_tags
              ON pd_project_tags( pd_tags_fk )
;
    CREATE INDEX pd_projects_fk___pd_project_tags
              ON pd_project_tags( pd_projects_fk )
;
    CREATE TABLE IF NOT EXISTS
      pd_files
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL, 
          ext                           TEXT NOT NULL, 
          relative_dir                  TEXT NOT NULL, 
          line_count                    INTEGER,

          pd_projects_fk                INTEGER NOT NULL,

          FOREIGN KEY ( pd_projects_fk )
            REFERENCES  pd_projects( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_projects_fk___pd_files
              ON pd_files( pd_projects_fk )
;
    CREATE TABLE IF NOT EXISTS
      pd_project_components
        (
          id                            INTEGER PRIMARY KEY,
          name                          TEXT NOT NULL,

          pd_projects_fk                INTEGER NOT NULL,

          FOREIGN KEY ( pd_projects_fk )
            REFERENCES  pd_projects( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_projects_fk___pd_project_components
              ON pd_project_components( pd_projects_fk )
;
    CREATE TABLE IF NOT EXISTS
      pd_project_subtags
        (
          id                            INTEGER PRIMARY KEY,

          pd_subtags_fk                 INTEGER NOT NULL,
          pd_projects_fk                INTEGER NOT NULL,

          FOREIGN KEY ( pd_subtags_fk )
            REFERENCES  pd_subtags( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE,

          FOREIGN KEY ( pd_projects_fk )
            REFERENCES  pd_projects( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_subtags_fk___pd_project_subtags
              ON pd_project_subtags( pd_subtags_fk )
;
    CREATE INDEX pd_projects_fk___pd_project_subtags
              ON pd_project_subtags( pd_projects_fk )
;
    CREATE TABLE IF NOT EXISTS
      pd_lines
        (
          id                            INTEGER PRIMARY KEY,
          line_number                   INTEGER,
          line_text                     TEXT,

          pd_files_fk                   INTEGER NOT NULL,

          FOREIGN KEY ( pd_files_fk )
            REFERENCES  pd_files( id )
              ON UPDATE CASCADE
              ON DELETE CASCADE
        )
;
    CREATE INDEX pd_files_fk___pd_lines
              ON pd_lines( pd_files_fk )
;

--=||=--
-- ===========================================================================
-- ===========================================================================
--     D R O P S
-- ===========================================================================
-- ===========================================================================

--=|Drop the tables.|=--

    DROP TABLE IF EXISTS pd_vendors;
    DROP TABLE IF EXISTS pd_roots;
    DROP TABLE IF EXISTS pd_tags;
    DROP TABLE IF EXISTS pd_projects;
    DROP TABLE IF EXISTS pd_subtags;
    DROP TABLE IF EXISTS pd_project_tags;
    DROP TABLE IF EXISTS pd_files;
    DROP TABLE IF EXISTS pd_project_components;
    DROP TABLE IF EXISTS pd_project_subtags;
    DROP TABLE IF EXISTS pd_lines;




--=||=--
-- ===========================================================================
-- ===========================================================================
--     L O A D I N G
-- ===========================================================================
-- ===========================================================================


--=|Create the loading tables.|=--

    CREATE TABLE IF NOT EXISTS
      load_pd_roots
        ( 
          id                  INTEGER,
          name                TEXT,
          path                TEXT,
          description         TEXT,
          load_code           INTEGER
        )
;

    CREATE TABLE IF NOT EXISTS
      load_pd_vendors
        ( 
          id                  INTEGER,
          name                TEXT,
          description         TEXT,
          load_code           INTEGER
        )
;

    CREATE TABLE IF NOT EXISTS
      load_pd_tags
        ( 
          id                  INTEGER,
          name                TEXT,
          description         TEXT,
          load_code           INTEGER
        )
;


-- Project

    CREATE TABLE IF NOT EXISTS
      load_pd_projects
        ( 
          id                  INTEGER,
          name                TEXT,
          dir                 TEXT,
          start_up            TEXT,
          version             TEXT,
          title               TEXT, 
          description         TEXT,
          authors             TEXT,
          tab_size            TEXT,
          language            TEXT,
          packager            TEXT,
          pd_vendors_fk       INTEGER,
          pd_roots_fk         INTEGER,

          pd_vendors_key      TEXT,
          pd_roots_key        TEXT,
          load_key            TEXT,
          load_code           INTEGER
        )
;
    CREATE TABLE IF NOT EXISTS
      load_pd_files
        ( 
          id                  INTEGER,
          name                TEXT,
          ext                 TEXT,
          relative_dir        TEXT, 
          line_count          INTEGER,
          pd_projects_fk      INTEGER,

          load_key            TEXT,
          load_code           INTEGER
        )
;
    CREATE TABLE IF NOT EXISTS
      load_pd_project_components
        ( 
          id                  INTEGER,
          name                TEXT,
          pd_projects_fk      INTEGER,

          load_key            TEXT,
          load_code           INTEGER
        )
;


--


    CREATE TABLE IF NOT EXISTS
      load_pd_subtags
        ( 
          id                  INTEGER,
          name                TEXT,
          pd_tags_fk          INTEGER,

          load_key            TEXT,
          load_code           INTEGER
        )
;


    CREATE TABLE IF NOT EXISTS
      load_pd_project_subtags
        ( 
          id                  INTEGER,
          pd_subtags_fk       INTEGER,
          pd_projects_fk      INTEGER,

          pd_subtags_key      TEXT,
          load_key            TEXT,
          load_code           INTEGER
        )
;
    CREATE TABLE IF NOT EXISTS
      load_pd_project_tags
        ( 
          id                  INTEGER,
          pd_tags_fk          INTEGER,
          pd_projects_fk      INTEGER,

          pd_tags_key         TEXT,
          load_key            TEXT,
          load_code           INTEGER
        )
;






--=|Drop the loading tables.|=--

    DROP TABLE IF EXISTS load_pd_files;
    DROP TABLE IF EXISTS load_pd_project_components;
    DROP TABLE IF EXISTS load_pd_project_subtags;
    DROP TABLE IF EXISTS load_pd_project_tags;
    DROP TABLE IF EXISTS load_pd_projects;
    DROP TABLE IF EXISTS load_pd_subtags;


--=|Set the IDs in the loading tables.|=--

   UPDATE load_pd_roots
   SET    id = 
	    ( 
          SELECT pd_roots.id
          FROM   pd_roots
          WHERE  pd_roots.name = load_pd_roots.name
     )
;

   UPDATE load_pd_vendors
   SET    id = 
	    ( 
          SELECT pd_vendors.id
          FROM   pd_vendors
          WHERE  pd_vendors.name = load_pd_vendors.name
        )
;

   UPDATE load_pd_tags
   SET    id = 
	    ( 
          SELECT pd_tags.id
          FROM   pd_tags
          WHERE  pd_tags.name = load_pd_tags.name
     )
;


--=|Append any new loading table records.|=--

    INSERT INTO 
      pd_roots
        ( 
          name,
          path,
          description
        )
        SELECT load_pd_roots.name,
               load_pd_roots.path,
               load_pd_roots.description
        FROM   load_pd_roots
        WHERE  load_pd_roots.id IS NULL
;
    INSERT INTO 
      pd_tags
        ( 
          name,
          description
        )
        SELECT load_pd_tags.name,
               load_pd_tags.description
        FROM   load_pd_tags
        WHERE  load_pd_tags.id IS NULL
;
    INSERT INTO 
      pd_vendors
        ( 
          name,
          description
        )
        SELECT load_pd_vendors.name,
               load_pd_vendors.description
        FROM   load_pd_vendors
        WHERE  load_pd_vendors.id IS NULL
;

